* lfax QBOX Support Module

* Module to execute a fax rx prog from qbox
* opion strings, file names and flags at end of file.
* also the flag no_dcf may be set to inhibit the dcffax entry point
* L.K.Wareham June 1994 V1.00

* If the modem autodetects a fax connect in data mode then
* the modem channel should be passed in a0 and the entry point confax called.
* 
* If the modem does not support autodetection then the entry point dcffax
* should be called on data connect message 'NO CARRIER'
* This entry only succeedes if the flag no_dcf is cleared.
* This can be used to disable this feature for modems that operate
* RELIABLY using confax.

io.open         EQU     $01
io.close        EQU     $02
fs.headr        EQU     $47
fs.load         EQU     $48
mt.cjob         EQU     $01
mt.activ        EQU     $0A
mt.frjob        EQU     $05

err.nc          EQU     -1
err.bp          EQU     -15
err.ni          EQU     -19

jb.a7           EQU     $5c
jb.end          EQU     $68


faxcon          EQU     $00010001       ;SB #1, used for lfax stderr
                                        ;and for stdin/stdout on dcffax entry
faxregs         REG     a0/a4

* Space for configurable strings and string macro

STRSPC   EQU    32

STRING$  MACRO    A
[.LAB]   DC.W     .E.[.L]-*-2
         DC.B     [A]
.E.[.L]  EQU      *
         ENDM


        XDEF    confax,dcffax


        SECTION CODE


dcffax
        movem.l faxregs,-(sp)
        move.b  no_dcf,d0
        bne     exitni          ;return not implemented
        moveq   #io.close,d0    ;modem chan prob marked EOF so close
        trap    #2              ;as lfax opens it's own in this case
        lea     modemchan,a1    ;store the modem/std chan here
        move.l  #faxcon,(a1)    ;pass the std channel defined in the equ
        lea     autofax,a1
        move.b  #0,(a1)         ;set entry type
        bra.s   start


confax
        movem.l faxregs,-(sp)
        lea     modemchan,a1    ;store the modem chan here
        move.l  a0,(a1)         ;as we pass this to lfax
        lea     autofax,a1
        move.b  #1,(a1)         ;set entry type

start
        lea     faxprog,a0
        moveq   #-1,d1          ;my chan
        moveq   #1,d3           ;old shared
        moveq   #io.open,d0     ;open
        trap    #2
        tst.l   d0
        bne     fail

        lea     filelen,a1
        move.w  #(fileend-filelen),d2
        moveq   #-1,d3
        moveq   #fs.headr,d0    ;read header
        trap    #3
        tst.l   d0
        bne     fail1

        move.l  a0,-(sp)        ;save chan ID
        move.b  filetyp,d0
        sub.b   #1,d0
        bne     failbp          ;if not exec
        move.l  filelen,d2      ;code space
        move.w  mfaxcmd,d3      ;length of command string
        add.w   #2+3*4+2,d3     ;and other stack stuff
        ext.l   d3
        add.l   filedat,d3      ;add to data space
        move.l  #0,a1
        moveq   #-1,d1          ;owned by me
        moveq   #mt.cjob,d0
        trap    #1
        tst.l   d0
        bne     fail1

        move.l  a0,a1           ;save pointer to new jobs code
        move.l  (sp)+,a0        ;get chan ID
        move.l  d1,-(sp)        ;save job ID

        move.l  jb.a7-jb.end(a1),a2     ;jobs stack pointer

        move.b  autofax,d0      ;different command line if autodetects
        beq.s   open_own
        lea     afaxcmd,a3
        bra.s   set_cmd

open_own
        lea     mfaxcmd,a3

set_cmd

        move.w  (a3),d2
        btst    #0,d2           ;odd ?
        beq.s   even
        addq.w  #1,d2
even    addq.w  #2,d2
        ext.l   d2
        sub.l   d2,a2           ;start of cmd string
        move.l  a2,a4           ;copy pointer

        move.w  (a3)+,(a4)+     ;set length

        subq.w  #2,d2


faxlp   subq.w  #2,d2           ;done string ?
        blt.s   eflp
        move.w  (a3)+,(a4)+
        bra.s   faxlp
eflp
        move.l  modemchan,-(a2) ;stdout
        move.l  #faxcon,-(a2)   ;stderr
        move.l  modemchan,-(a2) ;stdin
        move.w  #3,-(a2)        ;number of channels

* a0=Ch ID and a1= start of jobs code

        move.l  a2,jb.a7-jb.end(a1)     ;set jobs sp

        move.l  filelen,d2      ;file length
        moveq   #-1,d3
        moveq   #fs.load,d0     ;load file
        trap    #3
        tst.l   d0
        bne.s   cleanjb

        moveq   #io.close,d0
        trap    #2

        move.l  (sp)+,d1        ;get job ID
        move.b  #8,d2           ;priority
        move.w  #-1,d3          ;wait to finish
        moveq   #mt.activ,d0    ;start job
        trap    #1

fail    movem.l (sp)+,faxregs
        rts

exitni  moveq   #err.ni,d0
        bra.s   fail

cleanjb move.l  d0,d4
        moveq   #io.close,d0
        trap    #2

        move.l  (sp)+,d1        ;get job ID
        move.l  #err.nc,d3
        moveq   #mt.frjob,d0
        trap    #1

        move.l  d4,d0
        bra.s   fail

fail1   move.l  d0,d4
        move.l  (sp)+,a0        ;get chan ID
        moveq   #io.close,d0
        trap    #2
        move.l  d4,d0
        bra.s   fail

failbp  moveq   #err.bp,d0
        bra.s   fail1

* set the programme name and directory

* set one ofthese strings if your modem does not respond to the DCD overide
* command in FAX mode



faxprog  STRING$ 'flp1_fax_QLrunlfax'            ;for a QL with QBOX ext driver 2

*faxprog STRING$ 'flp1_fax_STrunlfax'            ;for SMS2/ST/QL

* or call lfax direct if your modem does respond to DCD overide in fax mode
*faxprog        STRING$ 'flp1_fax_lfax'

        DS.B   STRSPC-(*-faxprog)

* set this flag if the modem autodetects a fax connect in data mode
no_dcf          dc.b    0

autofax         dc.b    0


mfaxcmd STRING$ {'-R -x'}               ;lfax single shot, open own channel
        DS.B   STRSPC-(*-mfaxcmd)

afaxcmd STRING$ {'-p -x'}               ;lfax passed channel connect msg rx
        DS.B   STRSPC-(*-afaxcmd)

* workspace
modemchan dc.l  0

filelen dc.l    0
fileacc dc.b    0
filetyp dc.b    0
filedat dc.l    0
        dcb.b   $3a-$0a+4,0
fileend

        END
